import torch


def masked_cross_entropy_for_value(
    logits: torch.Tensor,
    target: torch.Tensor,
    pad_idx: int = 0,
) -> torch.Tensor:  # loss_gen
    mask = target.ne(pad_idx)
    logits_flat = logits.view(-1, logits.size(-1))
    log_probs_flat = torch.log(logits_flat)
    target_flat = target.view(-1, 1)
    losses_flat = -torch.gather(log_probs_flat, dim=1, index=target_flat)
    losses = losses_flat.view(*target.size())
    losses = losses * mask.float()
    loss = losses.sum() / (mask.sum().float())
    return loss
